/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

### File Specific:
   main-programmer: Patrick Boenzli
   co-programmer: ...
*/

#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_COLLISION_DETECTION

#include "cd_engine.h"
#include "obb_tree.h"
#include "debug.h"

#include "model.h"
#include "world_entity.h"
#include "terrain.h"
// #include "player.h"

#include "spatial_separation.h"
#include "quadtree.h"
#include "quadtree_node.h"



using namespace std;


/**
 *  standard constructor
 */
CDEngine::CDEngine ()
{
  this->setClassID(CL_CD_ENGINE, "CDEngine");
}


/**
 *  the singleton reference to this class
 */
CDEngine* CDEngine::singletonRef = NULL;


/**
 *  standard deconstructor
 */
CDEngine::~CDEngine ()
{
  CDEngine::singletonRef = NULL;
}


/**
 *
 */
void CDEngine::checkCollisions(std::list<WorldEntity*>& list1, std::list<WorldEntity*>& list2)
{
  BVTree* tree;
  std::list<WorldEntity*>::iterator entity1, entity2, pre1, pre2;
  PRINTF(5)("checking for collisions\n");

  pre1 = list1.begin();
  while (pre1 != list1.end())
  {
    entity1 = pre1++;
    if( likely((*entity1) != this->terrain))
    {
      pre2 = list2.begin();
      while (pre2 != list2.end())
      {
        entity2 = pre2++;
        if( likely((*entity2) != this->terrain))
        {
          PRINTF(4)("checking object %s against %s\n", (*entity1)->getName(), (*entity2)->getName());
          tree = (*entity1)->getOBBTree();
          if( likely(tree != NULL) && (*entity2)->getOBBTree() != NULL)
            tree->collideWith(*entity1, *entity2);
        }
      }
    }
  }
}


/**
 *  this checks the collisions with the ground
 */
void CDEngine::checkCollisionGround()
{
  if( likely( this->terrain != NULL))
  {
    Quadtree* q = dynamic_cast<Terrain*>(this->terrain)->ssp->getQuadtree();

//    QuadtreeNode* n = q->getQuadtreeFromPosition(this->player->getAbsCoor());
  }
  //sTriangleExt* tri = q->getTriangleFromPosition(this->player->getAbsCoor());
}


/**
 * some debug output on the class
 */
void CDEngine::debug()
{
  PRINT(0)("\n=============================| CDEngine::debug() |===\n");
  PRINT(0)("=  CDEngine: Spawning Tree Start\n");
  //this->rootTree->debug();
  PRINT(0)("=  CDEngine: Spawning Tree: Finished\n");
  PRINT(0)("=======================================================\n");

}


/**
 * this spawns a tree for debug purposes only
 */
void CDEngine::debugSpawnTree(int depth, sVec3D* vertices, int numVertices)
{
//   if ( this->rootTree == NULL)
//     this->rootTree = new OBBTree(depth, vertices, numVertices);
}


void CDEngine::drawBV(const std::list<WorldEntity*>& drawList ) const
{
  std::list<WorldEntity*>::const_iterator entity;
  for (entity = drawList.begin(); entity != drawList.end(); entity++)
    (*entity)->drawBVTree(1, 226);
}

/**
 * this draws the debug spawn tree
 */
void CDEngine::debugDraw(int depth, int drawMode)
{
  if(this-> rootTree != NULL)
    this->rootTree->drawBV(depth, drawMode);
}
